<?php
namespace Opencart\Catalog\Controller\Common;
class Kargonomi extends \Opencart\System\Engine\Controller {
    /**
     * Yeni bir endpoint eklemek istenirse;
     *      - $methods dizisinin key kısmına action name (endpointe parametre olarak dışarıdan gelen değer. Örn: action = order_list)
     *      $methods dizisinin value kısmına ise bu controllerdaki metod ismini (fonksiyon) yazılması gerekiyor.
     *      - Daha sonra oluşturulan yeni metod ile endpoint çalışmaya başlıyor.
     * 
     */

    public function index() {
        $methods = [
            'orders' => 'orders',
            'order_statuses' => 'orderStatuses',
            'add_history_to_order' => 'addHistoryToOrder',
        ];

        $token = $this->request->get['token'] ?? '';
        $username = $this->request->get['username'] ?? '';

        // Token'ı api_session tablosundan doğruluyoruz
        $api_info = $this->db->query("
            SELECT * FROM " . DB_PREFIX . "api 
            WHERE `key` = '" . $this->db->escape($token) . "' 
            AND username = '" . $this->db->escape($username) . "'
        ");

        if ($api_info->num_rows === 0) {
            // Geçersiz API bilgileri
            $this->response->addHeader('HTTP/1.1 403 Forbidden');
            $this->response->setOutput(json_encode(['error' => 'Invalid API Key'],JSON_PRETTY_PRINT));
            return;
        }

        if (!isset($this->request->get['action'])) {
            $this->response->addHeader('Content-Type: application/json');
            $this->response->addHeader('HTTP/1.1 404 Not Found');
            $this->response->setOutput(json_encode(['error' => 'Action not found']));
            return;
        }

        $action = $this->request->get['action'] ?? '';

        if (isset($methods[$action])) {
            $methodName = $methods[$action];
            return call_user_func([$this, $methodName]);
        } else {
            $this->response->addHeader('Content-Type: application/json');
            $this->response->addHeader('HTTP/1.1 400 Bad Request');
            $this->response->setOutput(json_encode(['error' => 'Invalid Action'],JSON_PRETTY_PRINT));
            return;
        }
    }

    public function orders()
    {
        $orderStatusId = isset($this->request->get['order_status_id']) && is_numeric($this->request->get['order_status_id']) ? (int)$this->request->get['order_status_id'] : null;
        $page = isset($this->request->get['page']) && (int)$this->request->get['page'] > 0 ? (int)$this->request->get['page'] : 1; // Varsayılan olarak 1
        $order_id = isset($this->request->get['order_id']) ? (int)$this->request->get['order_id'] : null;
        $limit = min((int)($this->request->get['per_page'] ?? 20), 100);  // per_page parametresi ile limit ayarlanıyor, max 100
        $start = ($page - 1) * $limit;

        // Sipariş verilerini getirmek için SQL sorgusu
        if ($order_id) {
            // Tek bir sipariş getir
            $order_query = $this->db->query("
                SELECT * FROM " . DB_PREFIX . "order
                WHERE order_id = '" . (int)$order_id . "'
            ");
            $orders = $order_query->rows;
        } else {
            // Çoklu siparişler için sorgu
            $sql = "
                SELECT * 
                FROM " . DB_PREFIX . "order
            ";

            if ($orderStatusId) {
                $sql .= " WHERE order_status_id = '" . (int)$orderStatusId . "'";
            }

            $sql .= " ORDER BY order_id DESC LIMIT " . (int)$start . ", " . (int)$limit;
            $order_query = $this->db->query($sql);
            $orders = $order_query->rows;
        }
        
        $defaultLanguageCode = $this->session->data['language'] ?? $this->language->get('code') ?? 'en';
        $languageQuery = $this->db->query("SELECT * FROM " . DB_PREFIX . "language WHERE code LIKE '%".$defaultLanguageCode."%' LIMIT 1");
        $languageId = $languageQuery->row['language_id'] ?? 1; //Default dil id'si 1

        $order_data = [];
        foreach ($orders as $order) {
            // Her siparişin ürünlerini ve product tablosundaki bilgileri çekiyoruz
            $productQuery = $this->db->query("
                SELECT op.*, p.product_id, p.sku, p.price as product_price, p.weight, p.length, p.width, p.height
                FROM " . DB_PREFIX . "order_product op
                JOIN " . DB_PREFIX . "product p ON op.product_id = p.product_id
                WHERE op.order_id = '" . (int)$order['order_id'] . "'
            ");

            $orderStatus = $this->db->query("
                SELECT *
                FROM " . DB_PREFIX . "order_status
                WHERE order_status_id = '" . (int)$order['order_status_id'] . "'
                AND language_id = '".$languageId."'
            ");
        
            $orderItems = [];
            
            foreach ($productQuery->rows as $product) {
                $orderItems[] = [
                    'order_product' => [
                        'order_product_id' => $product['order_product_id'],
                        'order_id' => $product['order_id'],
                        'product_id' => $product['product_id'],
                        'name' => $product['name'],
                        'model' => $product['model'],
                        'quantity' => $product['quantity'],
                        'price' => $product['price'],
                        'total' => $product['total'],
                        'tax' => $product['tax'],
                        'reward' => $product['reward'],
                    ],
                    'product_details' => [
                        'product_id' => $product['product_id'],
                        'sku' => $product['sku'],
                        'price' => $product['product_price'],
                        'weight' => $product['weight'],
                        'length' => $product['length'],
                        'width' => $product['width'],
                        'height' => $product['height']
                    ]
                ];
            }
        
            // Ürünleri sipariş verisine ekle
            $order['order_items'] = $orderItems;
            $order['order_status'] = $orderStatus->rows;
            $order_data[] = $order;
        }

        // Toplam sipariş sayısını çekme (pagination için) eğer order_id yoksa
        if (!$order_id) {
            $total_sql = "SELECT COUNT(*) as total FROM " . DB_PREFIX . "order";
            if ($orderStatusId) {
                $total_sql .= " WHERE order_status_id = '" . (int)$orderStatusId . "'";
            }

            $total_orders_query = $this->db->query($total_sql);
            $total_orders = $total_orders_query->row['total'];
        } else {
            $total_orders = count($orders); // Tek sipariş olduğunda toplam 1 olur
        }

        // JSON yanıtı
        $response = array(
            'page' => $page,
            'limit' => $limit,
            'total_orders' => $total_orders,
            'data' => $order_data  // Siparişler ve ürünler ayrı ayrı çekildi
        );

        $this->response->addHeader('Content-Type: application/json');
        $this->response->setOutput(json_encode($response, JSON_PRETTY_PRINT));
    }

    public function orderStatuses()
    {
        $defaultLanguageCode = $this->session->data['language'] ?? $this->language->get('code') ?? 'en';
        $languageQuery = $this->db->query("SELECT * FROM " . DB_PREFIX . "language WHERE code LIKE '%".$defaultLanguageCode."%' LIMIT 1");

        if(isset($languageQuery->row['language_id'])) {
            $query = $this->db->query("SELECT * FROM " . DB_PREFIX . "order_status WHERE language_id = ".$languageQuery->row['language_id']." ORDER BY order_status_id ASC");
        }else {
            $query = $this->db->query("SELECT * FROM " . DB_PREFIX . "order_status ORDER BY order_status_id ASC");
        }

        $orderStatusesData = $query->rows;
        $response = array(
            'page' => 1,
            'limit' => 100,
            'total_orders' => count($orderStatusesData),
            'data' => $orderStatusesData
        );

        $this->response->addHeader('Content-Type: application/json');
        $this->response->setOutput(json_encode($response, JSON_PRETTY_PRINT));
    }

    public function addHistoryToOrder()
    {
        $json = array();
        
        // order_id validation (must be a positive number)
        if (!isset($this->request->post['order_id']) || !is_numeric($this->request->post['order_id']) || (int)$this->request->post['order_id'] <= 0) {
            $json['error'] = 'Invalid order_id!';
            $this->response->addHeader('Content-Type: application/json');
            $this->response->addHeader('HTTP/1.1 400 Bad Request');
            $this->response->setOutput(json_encode($json));
            return;
        }
        
        // order_status_id validation (must be a positive number)
        if (!isset($this->request->post['order_status_id']) || !is_numeric($this->request->post['order_status_id']) || (int)$this->request->post['order_status_id'] <= 0) {
            $json['error'] = 'Invalid order_status_id!';
            $this->response->addHeader('Content-Type: application/json');
            $this->response->addHeader('HTTP/1.1 400 Bad Request');
            $this->response->setOutput(json_encode($json));
            return;
        }

        // If validation is successful, add new order history
        $this->load->model('checkout/order');

        // Raw query to insert into order_history table
        $this->db->query("INSERT INTO " . DB_PREFIX . "order_history SET 
            order_id = '" . (int)$this->request->post['order_id'] . "', 
            order_status_id = '" . (int)$this->request->post['order_status_id'] . "', 
            notify = '" . (isset($this->request->post['notify']) ? (int)$this->request->post['notify'] : 0) . "', 
            comment = '" . $this->db->escape(isset($this->request->post['comment']) ? $this->request->post['comment'] : '') . "', 
            date_added = NOW()");

        // Son eklenen kaydın ID'sini al
        $order_history_id = $this->db->getLastId();

        // Eklenen kaydı tekrar sorgula
        $query = $this->db->query("SELECT * FROM " . DB_PREFIX . "order_history WHERE order_history_id = '" . (int)$order_history_id . "'");

        $order_query = $this->db->query("SELECT * FROM " . DB_PREFIX . "order WHERE order_id = '" . (int)$this->request->post['order_id'] . "'");
    
        if ($order_query->num_rows) {
            // Order'ın durumu güncellenir
            $this->db->query("UPDATE " . DB_PREFIX . "order SET order_status_id = '" . (int)$this->request->post['order_status_id'] . "', date_modified = NOW() WHERE order_id = '" . (int)$this->request->post['order_id'] . "'");
        }

        // Success message and the newly added record
        $json['success'] = 'Order history added successfully!';
        $json['order_history'] = $query->row;

        // Set the response headers and status code
        $this->response->addHeader('Content-Type: application/json');
        $this->response->addHeader('HTTP/1.1 201 Created');
        $this->response->setOutput(json_encode($json));
    }
}
